@@ -84,6 +84,12 @@ sub label {
   $d;
 }
 
+sub notes {
+  my $self = shift;
+  return unless exists $self->{note};
+  @{$self->{note}};
+}
+
 sub note {
   my $self = shift;
   my $d = $self->{note};
@@ -91,6 +97,11 @@ sub note {
   $d;
 }
 
+sub add_note {
+  my $self = shift;
+  push @{$self->{note}},shift;
+}
+
 sub target {
   my $self = shift;
   my $d = $self->{target};
