@@ -24,6 +24,7 @@ disclaimers of warranty.
 =cut
 
 use strict;
+use Data::Dumper;
 
 sub new {
   my ($class, $defs) = @_;
@@ -45,7 +46,34 @@ sub new {
   return $self;
 }
 
-sub init       {};
+#########
+# Called at adaptor initialisation
+#
+sub init {};
+
+#########
+# mapmaster for this source. overrides configuration 'mapmaster' setting
+#
+sub mapmaster {}
+
+#########
+# hook for optimising results to be returned.
+# default - do nothing
+# Not necessary for most circumstances, but useful for deciding on what sort
+# of coordinate system you return the results if more than one type is available.
+#
+sub init_segments {}
+
+#########
+# returns a list of valid segments that this adaptor knows about
+#
+sub known_segments {}
+
+#########
+# returns a list of valid segments that this adaptor knows about
+#
+sub segment_version {}
+
 
 sub dsn {
   my $self = shift;
@@ -120,9 +148,11 @@ sub das_dsn {
   for my $adaptor ($self->config->adaptors()) {
     my $dsn        = $adaptor->dsn();
     my $dsnversion = $adaptor->dsnversion();
+    my $mapmaster  = $adaptor->mapmaster() || $adaptor->config->{'mapmaster'} || "http://$host:$port/das/$dsn/";
+
     $content .= qq(  <DSN>
     <SOURCE id="$dsn" version="$dsnversion">$dsn</SOURCE>
-    <MAPMASTER>http://$host:$port/das/$dsn/</MAPMASTER>
+    <MAPMASTER>$mapmaster</MAPMASTER>
     <DESCRIPTION>$dsn feature annotation</DESCRIPTION>
   </DSN>\n);
   }
@@ -172,12 +202,13 @@ sub close_dasgff {
 }
 
 #########
-# hook for optimising results to be returned.
-# default - do nothing
-# Not necessary for most circumstances, but useful for deciding on what sort
-# of coordinate system you return the results if more than one type is available.
+# unknown segment error response
 #
-sub init_segments {
+sub unknown_segment {
+  my ($self, $seg) = @_;
+  my $error = "";
+  $error .=  qq(    <UNKNOWNSEGMENT id="$seg" />\n);
+  return($error);
 }
 
 #########
@@ -186,28 +217,39 @@ sub init_segments {
 sub das_features {
   my ($self, $opts) = @_;
   my $response      = "";
-
+  
   $self->init_segments($opts->{'segments'});
-
+  
   #########
-  # straight features
+  # features on segments
   #
   for my $seg (@{$opts->{'segments'}}) {
+    
     my ($seg, $coords) = split(':', $seg);
     my ($start, $end)  = split(',', $coords||"");
     my $segstart       = $start || $self->start($seg) || "";
     my $segend         = $end   || $self->end($seg)   || "";
-    $response         .= qq(    <SEGMENT id="$seg" version="1.0" start="$segstart" stop="$segend">\n);
-
+    
+    if ( $self->known_segments() ){
+      unless (grep /$seg/ , $self->known_segments() ){
+	$response  .= $self->unknown_segment($seg);
+	next;
+      }
+    }
+    
+    my $segment_version = $self->segment_version() ||  "1.0";
+    $response          .= qq(    <SEGMENT id="$seg" version="$segment_version" start="$segstart" stop="$segend">\n);
+    
     for my $feature ($self->build_features({
 					    'segment'   => $seg,
 					    'start'     => $start,
 					    'end'       => $end,
-					   }  )) {
+					   })) {
       my $start    = $feature->{'start'}        || "0";
       my $end      = $feature->{'end'}          || "0";
       my $note     = $feature->{'note'}         || "";
       my $id       = $feature->{'id'}           || "";
+      my $label    = $feature->{'label'}        || $id;
       my $type     = $feature->{'type'}         || "";
       my $method   = $feature->{'method'}       || "";
       my $group    = $feature->{'group'}        || "";
@@ -222,8 +264,7 @@ sub das_features {
       my $subparts = $feature->{'typesubparts'}    || "no";
       my $supparts = $feature->{'typessuperparts'} || "no";
       my $ref      = $feature->{'typesreference'}  || "no";
-#      $response   .= qq(    <SEGMENT id="$seg" version="1.0" start="$start" stop="$end">\n);
-      $response   .= qq(      <FEATURE id="$id" label="$id">\n);
+      $response   .= qq(      <FEATURE id="$id" label="$label">\n);
       $response   .= qq(        <TYPE id="$type" $cat reference="$ref" subparts="$subparts" superparts="$supparts">$type</TYPE>\n);
       $response   .= qq(        <METHOD id="$method">$method</METHOD>\n) if($method ne "");
       $response   .= qq(        <START>$start</START>\n);
@@ -236,28 +277,40 @@ sub das_features {
       $response   .= qq(        <LINK href="$link">$linktxt</LINK>\n)    if($link  ne "");
       $response   .= qq(        <TARGET id="$id" start="$tst" stop="$tend" />\n) if($tst ne "" && $tend ne "");
       $response   .= qq(      </FEATURE>\n);
-#      $response   .= qq(    </SEGMENT>\n);
     }
     $response .= qq(    </SEGMENT>\n);
   }
 
   #########
-  # features by id
+  # features by specific id
   #
+  my $error_feature = 1;
+  
   for my $fid (@{$opts->{'features'}}) {
-    for my $feature ($self->build_features({
-					    'feature'   => $fid,
-					   })) {
+  
+    my @f = $self->build_features({
+				   'feature_id' => $fid,
+				  });
+    unless (@f) {
+      $response .= $self->error_feature($fid);
+      next;
+    }
+    
+    for my $feature (@f) {
       my $seg      = $feature->{'segment'}      || "";
+      my $segstart = $feature->{'segment_start'}        || $feature->{'start'}  || "";
+      my $segend   = $feature->{'segment_end'}          || $feature->{'end'}	|| "";
+      my $segver   = $feature->{'segment_version'}      || "1.0";
       my $start    = $feature->{'start'}        || "";
       my $end      = $feature->{'end'}          || "";
       my $note     = $feature->{'note'}         || "";
       my $id       = $feature->{'id'}           || "";
+      my $label    = $feature->{'label'}        || $id;
       my $type     = $feature->{'type'}         || "unknown";
       my $method   = $feature->{'method'}       || "";
       my $group    = $feature->{'group'}        || "";
       my $score    = $feature->{'score'}        || "";
-      my $ori      = $feature->{'ori'}          || "0";
+      my $ori      = $feature->{'ori'}          || $feature->{'orientation'} ||  "0";
       my $phase    = $feature->{'phase'}        || "";
       my $link     = $feature->{'link'}         || "";
       my $linktxt  = $feature->{'linktxt'}      || $link;
@@ -267,8 +320,8 @@ sub das_features {
       my $subparts = $feature->{'typesubparts'}    || "no";
       my $supparts = $feature->{'typessuperparts'} || "no";
       my $ref      = $feature->{'typesreference'}  || "no";
-      $response   .= qq(    <SEGMENT id="$seg" version="1.0" start="$start" stop="$end">\n);
-      $response   .= qq(      <FEATURE id="$id" label="$id">\n);
+      $response   .= qq(    <SEGMENT id="$seg" version="$segver" start="$segstart" stop="$segend">\n);
+      $response   .= qq(      <FEATURE id="$id" label="$label">\n);
       $response   .= qq(        <TYPE id="$type" $cat reference="$ref" subparts="$subparts" superparts="$supparts">$type</TYPE>\n);
       $response   .= qq(        <METHOD id="$method">$method</METHOD>\n) if($method ne "");
       $response   .= qq(        <START>$start</START>\n);
@@ -282,11 +335,23 @@ sub das_features {
       $response   .= qq(        <TARGET id="$id" start="$tst" stop="$tend" />\n) if($tst ne "" && $tend ne "");
       $response   .= qq(      </FEATURE>\n);
       $response   .= qq(    </SEGMENT>\n);
+
     }
   }
+  
   return $response;
 }
 
+#########
+# unknown feature error response
+#
+sub error_feature {
+  my ($self, $f) = @_;
+  qq(    <SEGMENT id="">
+      <UNKNOWNFEATURE id="$f" />
+    </SEGMENT>\n);
+}
+
 #########
 # open dna/sequence response
 #
@@ -319,7 +384,6 @@ sub das_dna {
     $response .= qq(  <SEQUENCE id="$seg" start="$segstart" stop="$segend" moltype="$moltype" version="1.0">\n);
     $response .= qq(    <DNA length="$len">\n      $seq\n);
     $response .= qq(    </DNA>\n);
-#    $response .= "$seq\n";
     $response .= qq(  </SEQUENCE>\n);
   }
   return $response;
@@ -346,22 +410,51 @@ sub close_dastypes {
 #
 sub das_types {
   my ($self, $opts) = @_;
-
+  
   my $response = "";
-  for my $seg (@{$opts->{'segments'}}) {
+  my @types    = ();
+  my $data     = {};
+  
+  unless (@{$opts->{'segments'}}){
+    $data->{'anon'} = [];
+    push (@{$data->{'anon'}},$self->build_types());
+
+  } else {
+    for my $seg (@{$opts->{'segments'}}) {
+      my ($seg, $coords) = split(':', $seg);
+      my ($start, $end)  = split(',', $coords||"");
+      my $segstart       = $start || $self->start($seg) || "";
+      my $segend         = $end   || $self->end($seg)   || "";
+      
+      $data->{$seg} = [];
+      @types = $self->build_types({
+				   'segment' => $seg,
+				   'start'   => $start,
+				   'end'     => $end,
+				  });
+      
+      push (@{$data->{$seg}}, @types);
+    }
+  }
+  
+  for my $seg (keys %{$data}) {
     my ($seg, $coords) = split(':', $seg);
-    my ($start, $end)  = split(',', $coords||"");
+    my ($start, $end)  = split(',', $coords || "");
     my $segstart       = $start || $self->start($seg) || "";
     my $segend         = $end   || $self->end($seg)   || "";
+    
+    if ($seg ne "anon") {
+      $response .= qq(  <SEGMENT id="$seg" start="$segstart" stop="$segend" version="1.0">\n);
 
-    $response .= qq(  <SEGMENT id="$seg" start="$segstart" stop="$segend" version="1.0">\n);
-    for my $type ($self->build_types({
-				      'segment' => $seg,
-				      'start'   => $start,
-				      'end'     => $end,
-				     })) {
-      my $method = qq(method="$type->{'method'}") if(defined $type->{'method'});
-      $response .= qq(    <TYPE id="$type->{'type'}" $method>$type->{'count'}</type>\n);
+    } else {
+      $response .= qq(  <SEGMENT version="1.0">\n);
+    }
+
+    for my $type (@{$data->{$seg}}) {
+      $type->{'count'} ||= "";
+      my $method         = qq(method="$type->{'method'}") if(defined $type->{'method'});
+      my $category       = qq(category="$type->{'category'}") if(defined $type->{'category'});
+      $response         .= qq(    <TYPE id="$type->{'type'}" $method $category>$type->{'count'}</type>\n);
     }
     $response .= qq(  </SEGMENT>\n);
   }
@@ -402,9 +495,11 @@ sub das_entry_points {
   my $self    = shift;
   my $content = "";
   
-  for my $seg ($self->build_entry_points()) {
-    my $subparts = $seg->{'subparts'} || "yes"; # default to yes here as we're giving entrypoints
-    $content .= qq(    <SEGMENT id="$seg->{'segment'}" size="$seg->{'length'}" subparts="$subparts" />\n);
+  for my $ep ($self->build_entry_points()) {
+    my $subparts = $ep->{'subparts'} || "yes"; # default to yes here as we're giving entrypoints
+    $content .= qq(    <SEGMENT id="$ep->{'segment'}" );
+	$content .= qq(size="$ep->{'length'}" );
+	$content .= qq(subparts="$subparts" />\n);
   }
   
   return $content;
